function ImSeq = Avi2ImSeq(filename)
%SYNOPSIS: Reads avi files of ImageType truecolor (24 bits rgb).
%
%SYNTAX: ImSeq = Avi2ImSeq(filename, h, w, len)
%    The loaded avi file filename ('name.avi').
%
%NOTE:
%Reads uncompressed files and files compressed with codes
%supported by matlab (see help aviread).
%
%FORMATS:
%Image sequence format: Greyscale [0;255].
%DIM = (height, width, length).
%Data format: input avi truecolor (24 bit) ImageType, output uint8.
%
%DETAILED DESCRIPTION:
%Reads truecolor (24 bit rgb) avi files and turns them into 
%grayscale image sequence to be used for test of deinterlacers.
%h, w, length is obtained from aviinfo(filename) as Height, 
%Width and NumFrames.

%Written by Sune Keller, ITU, Copenhagen, DK, 2004-01-09 as part of a
%Master Thesis.
%Version: 1.6, last revision: 2006-07-17: loop 1:len not 1:len-1
%-----------------------------------------------------------------------

info = aviinfo(filename)

h = info.Height;
w = info.Width;
len = info.NumFrames;

mov = aviread(filename); %mov is a matlab MOVIE.

ImSeq = zeros(h,w,len,'uint8');

for i = 1:len
    
    Im = frame2im(mov(i)); %Reading frames from mov one by one.
    
    %Im converted from uint8 to double for rgb to grayscale conversion.
    Im = double(Im);   
    Im = uint8((Im(:,:,1) + Im(:,:,2) + Im(:,:,3))/3);
    
    ImSeq(:,:,i) = Im;
    
    if (mod(i,50) == 0)
        fprintf(1,'frame no. %d finished\n',i);
    end
end