function Y = FRescale(X, Mn, Nn)
%RESCALE(X,Mn,Nn) rescales 2D array X to 2D array Y of dimension MnxNn
%
% uses very certainly the same principle as pnmrescale!
[M,N] = size(X);
% start by the N dimension
if Nn == N
     % nothing to do!
     Y1 = X;
else
     Y1 = zeros(M,Nn);
     unused_input = N*Nn;
     base_input = Nn;
     remainder = N;
     s = 1; t = 1;

     while unused_input > 0
         used = min(remainder, base_input);
         Y1(:,t) = Y1(:,t) + X(:,s)*used;
         unused_input = unused_input - used;
         base_input = base_input - used;
         remainder = remainder - used;

         % remainder = 0 means that output at Y1(:,t) is ready
         if remainder == 0
             Y1(:,t) = Y1(:,t)/N;
             t = t + 1;
             remainder = N;
         end

         % base_input = 0 means that we have exhausted input X(:,s), so
         % fetch next input and reset base_input
         if base_input == 0
             s = s + 1;
             base_input = Nn;
         end
     end
end

% then the M dimension
if Mn == M
     Y = Y1;
else
     Y = zeros(Mn,Nn);
     unused_input = M*Mn;
     base_input = Mn;
     remainder = M;
     s = 1; t = 1;

     while unused_input > 0
         used = min(remainder, base_input);
         Y(t,:) = Y(t,:) + Y1(s,:)*used;
         unused_input = unused_input - used;
         base_input = base_input - used;
         remainder = remainder - used;

         % remainder = 0 means that output at Y1(:,t) is ready
         if remainder == 0
             Y(t,:) = Y(t,:)/M;
             t = t + 1;
             remainder = M;
         end

         % base_input = 0 means that we have exhausted input Y1(s,:), so
         % fetch next input and reset base_input
         if base_input == 0
             s = s + 1;
             base_input = Mn;
         end
     end
end
