function ImSeq2avi(ImSeq, filename, fps)
%SYNOPSIS: Creates an avi file from an image sequence.
%
%SYNTAX: ImSeq2avi(ImSeq, filename, fps)
%    ImSeq is the sequence to converted into a avi file named
%    filename ('myMovie.avi') at fps frames per second.
%
%FORMATS:
%Image sequence format: Greyscale [0;255].
%DIM = (height, width, length).
%Data format: input  uint8,
%output uncompressed avi of ImageType truecolor (24 bit rgb).
%
%DETAILED DESCRIPTION:
%Makes an uncompressed truecolor avi-file via rgb conversion of
%the greyscale ImSeq.
%fps is the number of frames per second in the output file.

%Written by Sune Keller, ITU, Copenhagen, DK, 2004-01-09 as part of a
%Master Thesis.
%Version: 1.2, last revision: 2004-05-05.
%-----------------------------------------------------------------------

h = size(ImSeq,1);
w = size(ImSeq,2);
length = size(ImSeq,3);

b = uint8(zeros(h,w,3));

for k = 1:length
    
    b(:,:,1) = ImSeq(:,:,k);
    b(:,:,2) = ImSeq(:,:,k);
    b(:,:,3) = ImSeq(:,:,k);

    mov(k) = im2frame(b);
end

movie2avi(mov,filename,'compression', 'None', 'fps', fps);

aviinfo(filename)