function [F,f] = SRconstraint1D(n,N)
% Building the matrix of conversion from n to N pixels (in 1D).
% n LR pixels transformed to N HR pixels by definition.
% If n > N you are by definition doing downscaling, not SR (although this
% constraint actually shows how to project back from HR to LR).
%
% Sune Keller, ITU Copenhagen, 25-08-2006.

%n and N should be integer numbers
if mod(n,1) | mod(N,1)
    error('n and N should be integers.')
end

% Downscaling can be done, just transpose output matrix.
% if n > N
%     nn = n;
%     n = N;
%     N = nn;
%     fprintf(1,'n and N switched, to do downscaling transpose the output matrix.\n');
% end
if n > N
    fprintf(1,'Note: You are now by definition doing downscaling not SR.\n')
end

% When used in 2D the call from 2D must be made aware of the changed n and
% N or things will be fucked up. I will just assume users are not stupid! I
% don't need this extra protection myself.
% g = gcd(n,N);
% if g ~= 1
%     n = n/g;
%     N = N/g;
%     fprintf(1,'A greatest common devisor of %d found.\n',g);
%     fprintf(1,'A %d x %d conversion matrix will be returned.\n',N,n);
% end

F = zeros(N,n);

count = n*N;

step = 1/n;

colSum = N/n;

r = 1;
c = 1;

for k = 1:(n*N)
    F(r,c) = F(r,c) + 1/n;
    if sum(F(r,:)) >= 1         % Using >= not == as e.g. 9 times 1/9 
        r = r + 1;              % by summing is not 1 in Matlab/double precision!!!
    end                         % By using >= one risks an error as the sum
    if sum(F(:,c)) >= (N/n)     % might be just under what it really should
        c = c + 1;              % be with ==. To solve change to e.g. >= (n-0.1)/n
    end                         % and >= (N-0.1)/n (0.1 better than eps), if
end                             % check shows something wrong in matrix.

f = n/N;
% F = F*(n/N); 

% f*F is the actual SR constraint that tells us how to project the HR
% pixels back onto the the LR pixel grid, but as I use it for the SR it is
% more locigal to look at F and not f*F as it shows the coeffients of how
% to go from LR to HR, how you would initialize and it also is what is used
% in the actual full algorithm for 2D variational SR.
