function [G,g] = SRconstraint2D(m,M,n,N)
% Building the matrix of conversion from m x n to M x N pixels (in 2D).
% m x n LR pixels transformed to M x N HR pixels by definition.
% If n > N  and/or m > M you are by definition doing downscaling, not SR
% (although this constraint actually shows how to project back from HR to LR).
%
% The output of this function is the SR cosntraint matrix R and can be used
% to initialize the HR values before doing the actual SR algorithm run.
%
% No matter whether m/M or n/N is the first or second coordinate or whether
% you use image or cartesian coordinates, just use the same when you build
% this as when you build the constraint coefficient matrix for the actual
% SR algorithm where you should also read the coeffients from it in the
% same way. Or do appropriate invertions.
%
% Sune Keller, ITU Copenhagen, 25-08-2006.

Fm = SRconstraint1D(m,M);
Fn = SRconstraint1D(n,N);

g = n*m/(N*M);
G = zeros(N*M,n*m);

for K = 1:N
    for k = 1:n
        A = Fn(K,k).*Fm;
        G(((K-1)*M+1):(K*M),((k-1)*m+1):(k*m)) = A;
    end
end
        