function HR = SRgenInit(LR,m,M,n,N, id)
% Function to init HR image sequence from LR sequence for SR
% algorithm. LR should be uint8 or data might be lost.
%
% m,M should be in the height (1. dim of LR/HR), n,N should be in the width.
%
% Sune Keller, ITU Copenhagen, 04/09 2006.

% LR = [1 2 3; 4 5 6];
% m = 2;
% M=3;
% n=3;
% N=4;
% id = 'ab';

if id(1:2) == 'SD'
    LR = fitToPAL(LR);
end
% a lot of controls could/should be made here similar to the above (which
% would be the most common mistake I would make from my SD test material):
% Dimensions and integer relations between sizes etc, but is is expected
% that the user is not dumb.

[sx, sy, sz] = size(LR);

[G,g] = SRconstraint2D(m,M,n,N);

HR = zeros(sx*M/m, sy*N/n, sz,'uint8');

[Sx, Sy, Sz] = size(HR);

for z = 1:sz %frame no
    for y = 1:(Sy/N) % block no in width
        for x = 1:(Sx/M) % block no in height

            % Index in the LR frame of the upper left corner of the
            % current LR block
            LRx = (x-1)*m+1;
            LRy = (y-1)*n+1;
            % The actual block extracted
            LRblock = LR(LRx:(LRx+m-1), LRy:(LRy+n-1), z);
            % Needed transform to do vector multipication with rows of G
            LRblock = double(LRblock(:));   
            
            % Looping through the current HR block and SR constraint G
            for k = 1:(M*N)

                % Internal indexes in the current HR block (from k)
                % (Matlab probably has a smart function for this...)
                HRbx = mod((k-1),M)+1;
                HRby = ceil(k/M);
                % and from that the global current indexes in the HR frame
                HRx = (x-1)*M + HRbx;
                HRy = (y-1)*N + HRby;

                % Finding the actual value for the current HR pixel
                HR(HRx,HRy, z) = G(k,:)*LRblock; % a round from double to uint8
            end
        end
    end
end
