function HRflow = SRgenInitFlow(LRflow,m,M,n,N, id)
% Function to init HR image sequence flow from LR sequence flow for SR
% algorithm. LR flow should be single (used in mexFile Var. SR) or data 
% might be lost.
%
% m,M should be in the height (1. dim of LR/HR), n,N should be in the width.
% Flow is indexed (x,y,time,flow component).
%
% Sune Keller, ITU Copenhagen, 05/09 2006.

if id(1:2) == 'SD'
    LRflow = fitToPALflow(LRflow, 'single');
    %LRflow(:,:,:,2) = fitToPALflow(LRflow(:,:,:,2), 'single');

else
    LRflow = single(LRflow);
end
% a lot of controls could/should be made here similar to the above (which
% would be the most common mistake I would make from my SD test material):
% Dimensions and integer relations between sizes etc, but is is expected
% that the user is not dumb.

[sx, sy, sz, sv] = size(LRflow);% sv=2 always, but if not included then sz is 2 times it true value 

% Change length of flow vectors to fit new HR frames
LRflow(:,:,:,1) = LRflow(:,:,:,1)*M/m;
LRflow(:,:,:,2) = LRflow(:,:,:,2)*N/n;

[G,g] = SRconstraint2D(m,M,n,N);% Not needed when using FRescale.

HRflow = zeros(sx*M/m, sy*N/n, sz, 2, 'single');

[Sx, Sy, Sz, Sv] = size(HRflow);
for v = 1:2 % flow vector components, x and y.
    for z = 1:sz % frame no

        HRflow(:,:,z,v) = FRescale(LRflow(:,:,z,v),Sx,Sy);
        
% The code below is correct, but for speed up by a factor of 50, I use
% Francois' resize function as the result is the same.
%
%         for y = 1:(Sy/N) % block no in width
%             for x = 1:(Sx/M) % block no in height
% 
%                 % Index in the LR frame of the upper left corner of the
%                 % current LR block
%                 LRx = (x-1)*m+1;
%                 LRy = (y-1)*n+1;
%                 % The actual block extracted
%                 LRblock = LRflow(LRx:(LRx+m-1), LRy:(LRy+n-1), z, v);
%                 % Needed transform to do vector multipication with rows of G
%                 LRblock = double(LRblock(:));
% 
%                 % Looping through the current HR block and SR constraint G
%                 for k = 1:(M*N)
% 
%                     % Internal indexes in the current HR block (from k)
%                     % (Matlab probably has a smart function for this...)
%                     HRbx = mod((k-1),M)+1;
%                     HRby = ceil(k/M);
%                     % and from that the global current indexes in the HR frame
%                     HRx = (x-1)*M + HRbx;
%                     HRy = (y-1)*N + HRby;
% 
%                     % Finding the actual value for the current HR pixel
%                     HRflow(HRx,HRy, z, v) = G(k,:)*LRblock; % a round from double to uint8
%                 end
%             end
%         end
    end
end
