/***************************************************************
 * The spatio-temporal motion compensated super resolver class *
 * Uses a variation on Chanas energy.                          *
 ***************************************************************/

#ifndef MCSRgen_H_LK889688GIUG89675G1402
#define MCSRgen_H_LK889688GIUG89675G1402

#ifdef MEX_FILE_FUNCTION
// for the matlab mex function, need the matlab plugin for cimg
#include <mex.h>
#define cimg_plugin "cimgmatlabplugin.h" 
#endif

#define cimg_lapack 0
#include <CImg.h>
using namespace cimg_library;


/*******************************************************
 * MCSRgen class: SR with a magnification factor of 2  *
 *******************************************************/
class MCSRgen {
    // algorithm parameters
	CImg<float> HR;			// the initialized HR sequence
	CImg<float> bHRflow,fHRflow;	// the backward and forward flows
	double lambdas;					// spatial smoothness weight
	double lambdat;					// flow lines smoothess weight
	double lambda0;					// SR constraint weight
	double mi, Ma;					// min and max intensity values allowed
    int counter;

	unsigned fixpoint;				// number of fixed point iterations 
	unsigned relax;					// max. number of relaxation iterations for each fixed point iteration.
	double convthres;				// convergence threshold for relaxation relative error

    unsigned M;						// height of the HR SR blocks
	unsigned N;						// width of the HR SR blocks
	CImg<float> F;					// the dicrect mapping/constraint/solution coeff. matrix

	unsigned nx, ny, nz;			// sequence dimensions;
	//CImg<float> HR;				// output high resolution SR sequence
	//CImg<float> Chi;				// data mask;
	CImg<float> A;					// spatial smoothness divergence coefs
	CImg<float> Bb,Bf;				// Flow smoothness divergence coefs.
	CImg<float> update;				// Reg. output, du, to be used in the actual recalc. (Ru'-u0)
	const double eps2;				// L^1 norm regularizer
	

public:
	MCSRgen(const CImg<float> & HR_, const CImg<float> & bHRflow_, const CImg<float> & fHRflow_,
				  double lambdas_, double lambdat_, unsigned fixpoint_, unsigned relax_,
				  double convthres_, unsigned M_, unsigned N_, const CImg<float> & F_);

	const CImg<float> & upscale();

private:
	inline static double sqr(double x) {return x*x;}
	void initSequence();                     // create the initial sequence
	void computeCoefficients();              // compute linear equation coefficients
	//void linearSolverPureInpainting();     // perform pure inpainting - not touching known values
											 // not needed for SR no original values left in Output
	void linearSolver();					 // perform inpainting and denoising --> SR with lock to org. values
	void SRconstraint(unsigned x, unsigned y, unsigned z); // imposes the SR constraint as backprojection keeping minmax values
};


#endif // #ifndef MCSRgen_H_LK889688GIUG89675G1402