/*********************************************************
 * Implementation of Simultaneous flow and intensity SR  *
 *										  			     *
 * NOTE: Version for Mosix!!!!!!!!!!!!!!!!!!!!!!		 *
 *********************************************************/

#include <iostream>
#include "SRsim.h"

//#ifndef MEXFILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
/*********************************
 * Check validity of a numerical *
 * value: throw a numerical      *
 * exception in case of problem. *
 *********************************/
void SRsim::checkValue(double val, unsigned x, unsigned y, unsigned z)
{
	if (val != val){

		try{
			throw NaNException(x, y, z);
		}
		catch(NaNException){
			mexPrintf("NaN occurred at level %i, imagesize %i x %i\n", clevel, cx, cy);
			mexPrintf("in fixed point iteration %i, relax iteration %i\n", cfix, crelax);
			mexPrintf("at position x = %i, y = %i and z = %i\n", x, y, z);
			mexPrintf("the value is: %f\n\n", val);
			NaNdetected = true;
			mexErrMsgTxt("fucked");
		}

	
	}
	else if (fabs(val) == HUGE_VAL){	
	mexPrintf("Huge value occurred at level %i, imagesize %i x %i\n", clevel, cx, cy);
	mexPrintf("in fixed point iteration %i, relax iteration %i\n", cfix, crelax);
	mexPrintf("at position x = %i, y = %i and z = %i\n\n", x, y, z);
	NaNdetected = true;
	}//throw INDException(x, y, z);
}
//#endif

/*******************************
 * 2x2 symmetric system solver 
 *******************************/
void SRsim::solve2x2(double a11, double a12, double a22, double b1, double b2, double &x, double &y)
{
	double invdet = 1.0/(a11*a22-a12*a12);
	x = invdet*(a22*b1 - a12*b2);
	y = invdet*(a11*b2 - a12*b1);
}


/*************************************
 * implement a resize  la pnmrescale
 * it's separable: starts by resizing 
 * columns and then rows.            
 *************************************/
/*void SRsim::resize(CImg<float> & source, CImg<float> & dest)
{
	unsigned xs = source.width, ys = source.height;
	unsigned xd = dest.width,   yd = dest.height;
	unsigned nz = source.depth, nv = source.dim; // we won't touch time and vectorial dimensions
	// create intermediary image, fill with 0's
	CImg<float> inter(xs, yd, nz, nv, 0.0);
	
	unsigned left = yd*ys, a = yd, b = ys;
	unsigned s = 0, d = 0, used = 0;

	// redim source columns from ys to yd pixels, save in sequence "inter"
	while (left > 0) {
		used = cimg::min(a,b); // fraction of the input we need
		left -= used; a -= used; b -= used;
		cimg_mapXZV(inter, x, z, v) inter(x, d, z, v) += source(x, s, z, v)*used;
		if (b == 0) {
			cimg_mapXZV(inter, x, z, v) inter(x, d, z, v) /= ys;
			b = ys; ++d;
		}
		if (a == 0) {
			a = yd; ++s;
		}
	}

	// redim inter rows from xs to xd pixels, save in to dest.
	dest.fill(0.0);
	left = xd*xs; a = xd; b = xs;
	s = 0; d = 0; used = 0;
	while (left > 0) {
		used = cimg::min(a,b);
		left -= used; a -= used; b -= used;
		cimg_mapYZV(dest, y, z, v) dest(d, y, z, v) += inter(s, y, z, v)*used;
		if (b == 0) {
			cimg_mapYZV(dest, y, z, v) dest(d, y, z, v) /= xs;
			b = xs; ++d;
		}
		if (a == 0) {
			a = xd;	++s;
		}
	}
	// we are done.
}*/

/**********************************************************
 * Returning data to Matlab			                      *
 **********************************************************/
const CImg<float> & SRsim::getfU() { return fU;}

const CImg<float> & SRsim::getbU() { return bU;}




/*********************************************************** 
 * getSeq is the "driver" routine						   * 
 ***********************************************************/
const CImg<float> & SRsim::getSeq() 
{
	cx = nx;
	cy = ny;

	// we start by initializimg the flow container to 0
	U = CImg<float>(cx, cy, nz-1, 2, 0.0);
	
	//outer looping alternating flow and intensity calc.
	for (unsigned ite = 1; ite <= outerIte; ++ite) {

		//Forward flow SR
		U = fU;
		computeIUDerivatives();			// Constant values
		fixedPointSolver();				// Fixed point solver which internally calls
										// comp. of nonlinear fixed values and the relax solver
		fU = U + dU;

		
		// Backward flow SR with reverse of sequence and flow
		const char axe='z';
        I.flip(axe);
		bU.flip(axe);
		U = bU;
		computeIUDerivatives();			// Constant values
		fixedPointSolver();				// Fixed point solver which internally calls
										// comp. of nonlinear fixed values and the relax solver 
		bU = U + dU;
		//flip back for use in intensity SR
		I.flip(axe);
		bU.flip(axe);


	// The fixed point solver, performing intensity SR 
 		for (unsigned i = 0; i < fixpointI; ++ i) {
			computeCoefficients();		// Constant and fixed nonlinear values
			linearSolver();				// Relax solver
		}
	
	}


	return I;
}


		/*------------------------------------------------
							FLOW METHODS
		 ------------------------------------------------*/


/****************************************************
 * compute the warped derivatives as well the fixed 
 * part of the flow jacobian dU
 ****************************************************/
void SRsim::computeIUDerivatives()
{
	S = CImg<float>(cx, cy, nz-1, 6);
	cimg_mapXYZ(S, x, y, z) {
		double ux = x + U(x,y,z,0), vy = y + U(x,y,z,1);

		// start by extracting the image values in the spatio-temporal neighborhood of interest
		// use 2 spatial neighborhoods, one centered at (x,y,t), and a similar one at (x+u,y+v,t+1) (x+u =: ux, y+v =: vy)
		double Iuww = I.cubic_pix2d(ux-2,vy,z+1), Iuw = I.cubic_pix2d(ux-1,vy,z+1);
		double Iuee = I.cubic_pix2d(ux+2,vy,z+1), Iue = I.cubic_pix2d(ux+1,vy,z+1);
		double Iuss = I.cubic_pix2d(ux,vy-2,z+1), Ius = I.cubic_pix2d(ux,vy-1,z+1);
		double Iunn = I.cubic_pix2d(ux,vy+2,z+1), Iun = I.cubic_pix2d(ux,vy+1,z+1);
		double Iunw = I.cubic_pix2d(ux-1,vy+1,z+1);
		double Iune = I.cubic_pix2d(ux+1,vy+1,z+1);
		double Iuse = I.cubic_pix2d(ux+1,vy-1,z+1);
		double Iusw = I.cubic_pix2d(ux-1,vy-1,z+1);
		double Iuc  = I.cubic_pix2d(ux,vy,z+1);

		double Iww  = I.neumann_pix2d(x-2,y,z), Iw = I.neumann_pix2d(x-1,y,z);
		double Iee  = I.neumann_pix2d(x+2,y,z), Ie = I.neumann_pix2d(x+1,y,z);
		double Iss  = I.neumann_pix2d(x,y-2,z), Is = I.neumann_pix2d(x,y-1,z);
		double Inn  = I.neumann_pix2d(x,y+2,z), In = I.neumann_pix2d(x,y+1,z);
		double Inw  = I.neumann_pix2d(x-1,y+1,z);
		double Ine  = I.neumann_pix2d(x+1,y+1,z);
		double Ise  = I.neumann_pix2d(x+1,y-1,z);
		double Isw  = I.neumann_pix2d(x-1,y-1,z);
		double Ic   = I(x,y,z);
		
		// from that define the different image derivatives
		double Iux = (Iuww - 8*Iuw + 8*Iue - Iuee)/12;
		double Ivy = (Iuss - 8*Ius + 8*Iun - Iunn)/12;
		double Ix  = (Iww - 8*Iw + 8*Ie -Iee)/12;
		double Iy  = (Iss - 8*Is + 8*In -Inn)/12;
		double It  = Iuc - Ic;
		double Ixt = Iux - Ix;
		double Iyt = Ivy - Iy;
		double Iuxx = Iuw + Iue - 2*Iuc;
		double Ivyy = Ius + Iun - 2*Iuc;
		double Iuxy = 0.25*(Iune + Iusw - Iunw - Iuse);

		// then the tensor coefficients
		S(x,y,z,0) = sqr(Iux) + gamma*(sqr(Iuxx) + sqr(Iuxy));   // S11
		S(x,y,z,1) = Iux*Ivy  + gamma*Iuxy*(Iuxx + Ivyy);        // S12
		S(x,y,z,2) = Iux*It   + gamma*(Iuxx*Ixt + Iuxy*Iyt);     // S13
		S(x,y,z,3) = sqr(Ivy) + gamma*(sqr(Iuxy) + sqr(Ivyy));   // S22
		S(x,y,z,4) = Ivy*It   + gamma*(Iuxy*Ixt + Ivyy*Iyt);     // S23;
		S(x,y,z,5) = sqr(It)  + gamma*(sqr(Ixt) + sqr(Iyt));     // S33;
	}
	
	//S.display("Tensor");

	JU = CImg<float>(cx, cy, nz-1, 6); // fixed part of the flow derivative. 
	cimg_mapXYZ(JU, x, y, z) { 

		JU(x,y,z,0) = (U.neumann_pix1d(x+1,y,z,0) - U.neumann_pix1d(x-1,y,z,0))/2.0;   // u1x
		JU(x,y,z,1) = (U.neumann_pix2d(x,y+1,z,0) - U.neumann_pix2d(x,y-1,z,0))/2.0;   // u1y
		JU(x,y,z,2) = (U.neumann_pix3d(x,y,z+1,0) - U.neumann_pix3d(x,y,z-1,0))/2.0;   // u1t
		JU(x,y,z,3) = (U.neumann_pix1d(x+1,y,z,1) - U.neumann_pix1d(x-1,y,z,1))/2.0;   // u2x
		JU(x,y,z,4) = (U.neumann_pix2d(x,y+1,z,1) - U.neumann_pix2d(x,y-1,z,1))/2.0;   // u2y
		JU(x,y,z,5) = (U.neumann_pix3d(x,y,z+1,1) - U.neumann_pix3d(x,y,z-1,1))/2.0;   // u2t

	}
}


/*************************************************
 * Compute the coefficients of the linear system *
 * that has to be solved at each fixed point     *
 * iteration.                                    *
 * We need the term \psi'_D(dUk^t S dUk)         *
 * and \psi'_R(||Juk + duk||_F^2)                *
 *************************************************/
void SRsim::computeEquationCoefs()
{
	// normally the Data term \psi'_D has been properly dimensioned.
	cimg_mapXYZ(Data, x, y, z) {
		double du1 = dU(x,y,z,0), du2 = dU(x,y,z,1);
		double tmp = sqr(du1)*S(x,y,z,0) + sqr(du2)*S(x,y,z,3) + S(x,y,z,5) + 
					2*(S(x,y,z,1)*du1*du2 + S(x,y,z,2)*du1 + S(x,y,z,4)*du2);
		Data(x,y,z) = 0.5/sqrt(fabs(tmp) + eps2);  
		//fabs is a 'hack' due to detection of (close to zero) negative values of tmp,
		//which is probably due to nummerical instabilities and/or computational impressions.

	}
	//Data.display("Data");
	CImg<float> JdU = CImg<float>(cx, cy, nz-1, 6); // variable part of the flow derivative. 
	cimg_mapXYZ(JdU, x, y, z) { 

		JdU(x,y,z,0) = (dU.neumann_pix1d(x+1,y,z,0) - dU.neumann_pix1d(x-1,y,z,0))/2.0;   // du1x
		JdU(x,y,z,1) = (dU.neumann_pix2d(x,y+1,z,0) - dU.neumann_pix2d(x,y-1,z,0))/2.0;   // du1y
		JdU(x,y,z,2) = (dU.neumann_pix3d(x,y,z+1,0) - dU.neumann_pix3d(x,y,z-1,0))/2.0;   // du1t
		JdU(x,y,z,3) = (dU.neumann_pix1d(x+1,y,z,1) - dU.neumann_pix1d(x-1,y,z,1))/2.0;   // du2x
		JdU(x,y,z,4) = (dU.neumann_pix2d(x,y+1,z,1) - dU.neumann_pix2d(x,y-1,z,1))/2.0;   // du2y
		JdU(x,y,z,5) = (dU.neumann_pix3d(x,y,z+1,1) - dU.neumann_pix3d(x,y,z-1,1))/2.0;   // du2t

	}
	
	cimg_mapXYZ(Regul, x, y, z) {
		double tmp = 0;
		// compute the square Frobenius norm of JU + JdU
		for (unsigned v = 0; v < 6; ++v) 
			tmp += sqr(JU(x,y,z,v)) + sqr(JdU(x,y,z,v));
		Regul(x,y,z) = 0.5/sqrt(tmp + eps2);
	}
	//Regul.display("Regul");
}


/***************************************************
 * The fixed point solver compute the flow update  *
 * dU at each resolution iteration.                *
 ***************************************************/
void SRsim::fixedPointSolver()
{

	// create the update flow, as a 0 field.
	dU = CImg<float>(cx, cy, nz-1, 2, 0.0);
	
	// create the Data and Regul arrays, with current spatial dimensions
	Data = CImg<float>(cx, cy, nz-1);
	Regul = CImg<float>(cx, cy, nz-1);

	// run a fixed amount of fixed point iterations 
	for (unsigned i = 0; i < fixpoint; ++i) {
		
		//for debugging:
		cfix = i;
		if(NaNdetected){ 
            mexPrintf("Breaking in fixedPointSolver\n");			
			break;
		}


		// get the coefs of the linear system to solve at that stage
		computeEquationCoefs();
		// and solve it
		relaxationSolver();
	}
}


/***********************************************
 * the linear solver: Block Gauss-Seidel or    *
 * block SOR.                                  *
 ***********************************************/
void SRsim::relaxationSolver() 
{
	unsigned idx = 0; // relaxation loop index, set to 0 to shut the compiler up
	double relError = 0; // relative error between to iterations = mean(||e^{n+1}-e^{n}||_2)

	double omega = 1.9;

//#ifndef MEX_FILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
//	CImgDisplay disp(U+dU,"flow");
//#endif

	for (idx = 0; idx < relax; ++idx) {

		//for debugging:
       
		if(NaNdetected){ 
            mexPrintf("Breaking in relaxSolver\n");			
			break;
		}
        crelax = idx;
		
		relError = 0.0;
		double dunew = 0, dvnew = 0;
		// loop over all the update flow sites
		cimg_mapXYZ(dU, x, y, z) {

			// extract all the values that will be used to compute the equations
			double A = Data(x,y,z);

			//Check values 4, for debugging:
			//if(x == 13 && y == 91 && z == 0 && clevel == 2 && cfix == 4 && crelax == 0 && cx == 88 && cy == 92){
			//	mexPrintf("about to check A\n");
			//checkValue(A, x, y, z);}


			//Check values 10 (with 9), for debugging:
			//checkValue(A, x, y, z);

			double S11 = S(x,y,z,0);
			double S12 = S(x,y,z,1);
			double S13 = S(x,y,z,2);
			double S22 = S(x,y,z,3);
			double S23 = S(x,y,z,4);
		
			double B  = Regul(x,y,z);
			double BW = Regul.neumann_pix1d(x-1,y,z), BE = Regul.neumann_pix1d(x+1,y,z);
			double BS = Regul.neumann_pix2d(x,y-1,z), BN = Regul.neumann_pix2d(x,y+1,z);
			double BB = Regul.neumann_pix3d(x,y,z-1), BA = Regul.neumann_pix3d(x,y,z+1);

			//double Bw = 0.5*(B + BW), Be = 0.5*(B + BE);
			//double Bs = 0.5*(B + BS), Bn = 0.5*(B + BN);
		    	//double Bb = 0.5*(B + BB), Ba = 0.5*(B + BA);
			double Bw = 2.0/(1.0/B + 1.0/BW), Be = 2.0/(1.0/B + 1.0/BE);
			double Bs = 2.0/(1.0/B + 1.0/BS), Bn = 2.0/(1.0/B + 1.0/BN);
			double Bb = 2.0/(1.0/B + 1.0/BB), Ba = 2.0/(1.0/B + 1.0/BA);

 
			double SB = Bw + Be + Bs + Bn + Bb + Ba;

			double uduw = U.neumann_pix1d(x-1,y,z,0) + dU.neumann_pix1d(x-1,y,z,0), udue = U.neumann_pix1d(x+1,y,z,0) + dU.neumann_pix1d(x+1,y,z,0);
			double udus = U.neumann_pix2d(x,y-1,z,0) + dU.neumann_pix2d(x,y-1,z,0), udun = U.neumann_pix2d(x,y+1,z,0) + dU.neumann_pix2d(x,y+1,z,0);
			double udub = U.neumann_pix3d(x,y,z-1,0) + dU.neumann_pix3d(x,y,z-1,0), udua = U.neumann_pix3d(x,y,z+1,0) + dU.neumann_pix3d(x,y,z+1,0);

			double vduw = U.neumann_pix1d(x-1,y,z,1) + dU.neumann_pix1d(x-1,y,z,1), vdue = U.neumann_pix1d(x+1,y,z,1) + dU.neumann_pix1d(x+1,y,z,1);
			double vdus = U.neumann_pix2d(x,y-1,z,1) + dU.neumann_pix2d(x,y-1,z,1), vdun = U.neumann_pix2d(x,y+1,z,1) + dU.neumann_pix2d(x,y+1,z,1);
			double vdub = U.neumann_pix3d(x,y,z-1,1) + dU.neumann_pix3d(x,y,z-1,1), vdua = U.neumann_pix3d(x,y,z+1,1) + dU.neumann_pix3d(x,y,z+1,1);

			double Uc  = U(x,y,z,0),  Vc  = U(x,y,z,1);
			double dUc = dU(x,y,z,0), dVc = dU(x,y,z,1);
		
			double a11 = A*S11 + alpha*SB, a12 = A*S12, a22 = A*S22 + alpha*SB;
			double C1 = alpha*(Bw*uduw + Be*udue + Bs*udus + Bn*udun + Bb*udub + Ba*udua -SB*Uc) - A*S13;
			double C2 = alpha*(Bw*vduw + Be*vdue + Bs*vdus + Bn*vdun + Bb*vdub + Ba*vdua -SB*Vc) - A*S23;


			// we have all the system coefficients: solve
			solve2x2(a11, a12, a22, C1, C2, dunew, dvnew);

			
			//Check values 2, for debugging:
			//checkValue(dunew, x, y, z);
			//checkValue(dvnew, x, y, z);
		
			
			
			dunew = (1.0-omega)*dUc + omega*dunew;
			dvnew = (1.0-omega)*dVc + omega*dvnew;
//#ifndef MEXFILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
			//Check values 1, for debugging:
			//checkValue(dunew, x, y, z);
			//checkValue(dvnew, x, y, z);
//#endif

			relError += sqr(dUc-dunew) + sqr(dVc-dvnew);
			dU(x,y,z,0) = dunew;
			dU(x,y,z,1) = dvnew;


		}
//#ifndef MEX_FILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
//		(U+dU).display(disp);
//#endif

		// compute the relative error after this pixels run
		relError = sqrt(relError)/(cx*cy*(nz-1.0));
		if (relError < convthres) {  // we have converged :-)
			// can add something clever here
			//mexPrintf("convergenge at level %i, relax %i, fixpoint %i\n", clevel, crelax, cfix);
			break;
		}
	}
	// could check wether idx == relax to now wether we have converged
}

		/*------------------------------------------------
						 INTENSITY METHODS
		 ------------------------------------------------*/

/********************************************************
 * Computes the coefficients that are used to solve the *
 * linear system at a given fixed point iteration.      *
 ********************************************************/
void SRsim::computeCoefficients()
{	
	CImg_3x3(I, float);
	// a very simple A : no half pixels computations here
	cimg_mapZ(I, z) cimg_map3x3(I, x, y, z, 0, I) {
		A(x,y,z) = 1.0/sqrt(0.25*(sqr(Inc - Ipc) + sqr(Icn - Icp)) + eps2);
	}
	
	// Here, I do not have much choice, do I?
	cimg_mapXYZ(Bf, x, y, z) {

		float Ic = I(x,y,z);
		float fxn = x + fU.dirichlet_pix3d(x,y,z,0),   fyn = y + fU.dirichlet_pix3d(x,y,z,1);
		Bf(x,y,z) = 1.0/sqrt(sqr(I.linear_pix3d(fxn,fyn,z+1) - Ic) + eps2);
		float fxp = x + bU.dirichlet_pix3d(x,y,z-1,0), fyp = y + bU.dirichlet_pix3d(x,y,z-1,1);
		Bb(x,y,z) = 1.0/sqrt(sqr(Ic - I.linear_pix3d(fxp,fyp,z-1)) + eps2);
	}
}

/**********************************************************
 * The following routine is the linear solvers used.      *
 **********************************************************/

void SRsim::linearSolver() 
{
	double relError = 0.0;
	double omega = 1.0;   // SOR overrelaxation "parameter"
	
//#ifndef MATLAB_MEX_FILE
//	std::cout << "Hello, Inpainting Denoising Solver!" << std::endl;
//	CImgDisplay disp(I,"scaled");
//#endif

	for (unsigned i = 0; i < relaxI; ++i) {
//#ifndef MATLAB_MEX_FILE
//	std::cout << "At iteration " << i << std::endl;
//#endif
		double mylambdat;
		relError = 0.0;
		float SA, Ac, Aw, An, Ae, As, SB, Bp, Bm, Sw, Sn, Se, Ss, Sm, Sp, Sc, fxn, fyn, fxp, fyp, Snew;//S0, Chic;
		
		// Controlling temporal reg. in first and last frame
		for (int z = 0; z < nz; z++) {  //x++ and ++x different from java, x=0, ++x still gives x=0 inside first loop
			if (z == 0 || z == nz-1) mylambdat = 0;
			else mylambdat = lambdat;

			cimg_mapXY(I, x, y) {

				// extract coefficients

				// first for the spatial diffusion
				Ac = A(x,y,z);
				//Aw = 0.5*(Ac + A.neumann_pix1d(x-1,y,z)); Ae = 0.5*(Ac + A.neumann_pix1d(x+1,y,z));
				//As = 0.5*(Ac + A.neumann_pix2d(x,y-1,z)); An = 0.5*(Ac + A.neumann_pix2d(x,y+1,z));
				Aw = 2.0/(1.0/Ac + 1.0/A.neumann_pix1d(x-1,y,z)); Ae = 2.0/(1.0/Ac + 1.0/A.neumann_pix1d(x+1,y,z));
				As = 2.0/(1.0/Ac + 1.0/A.neumann_pix2d(x,y-1,z)); An = 2.0/(1.0/Ac + 1.0/A.neumann_pix2d(x,y+1,z));
				SA = Aw + Ae + As + An;

				// then the temporal diffusion
				Bm = Bb(x,y,z); Bp = Bf(x,y,z);
				SB = Bm + Bp;

				// get forward and backward displacements
				fxn = x + fU.dirichlet_pix3d(x,y,z,0);   fyn = y + fU.dirichlet_pix3d(x,y,z,1);
				fxp = x + bU.dirichlet_pix3d(x,y,z-1,0); fyp = y + bU.dirichlet_pix3d(x,y,z-1,1);

				// extract intensity values in the spatial-flow neighborhood
				Sc = I(x,y,z); 
				Sw = I.neumann_pix1d(x-1,y,z); Se = I.neumann_pix1d(x+1,y,z);
				Ss = I.neumann_pix2d(x,y-1,z); Sn = I.neumann_pix2d(x,y+1,z);
				Sm = I.linear_pix3d(fxp, fyp, z-1); Sp = I.linear_pix3d(fxn, fyn, z+1);

				// compute the relaxation at that pixel     
				Snew = (lambdas*(Aw*Sw + Ae*Se + As*Ss + An*Sn) + mylambdat*(Bm*Sm + Bp*Sp))/(lambdas*SA + mylambdat*SB);
				// Snew becomes the overrelaxation
				//Snew = (1-omega)*Sc + omega*Snew;

				// update relative error and image pixel update value;
				relError += sqr(Snew - Sc);
				update(x,y,z) = I(x,y,z) - Snew;
				
			}
		}
		//mexPrintf("x%i",M);
		//iterationg through the frames block by block to impose the SR constraint by backprojection
		for (unsigned z = 0; z < nz; z++){
			for (unsigned y = 0; y < ny; y += M){
                for (unsigned x = 0; x < nx; x += N){

					SRconstraint(x,y,z);
				}
			}
		}

		//mexPrintf("number of recursive calls in total: %i\n",counter);
		
//#ifndef MATLAB_MEX_FILE
//		I.display(disp);
//#endif
		relError = sqrt(relError)/(nx*ny*nz);
		if (relError < convthres) break;
	}
}
/**********************************************************
 * The following routine imposes the SR constraint by     *
 * backprojection in each 2x2 neighborhood of support.    *
 **********************************************************/

void SRsim::SRconstraint(unsigned x, unsigned y, unsigned z)
{
		
	CImg<float> uBlock(N,M); // width, height
	//uBlock.fill(0.0); 
	unsigned curPixVec = 0;
	bool minMaxViolated = false;
	
	//iterating to update each pixel in the block
	for(unsigned bx = 0; bx < N; bx++){				//width
		for(unsigned by = 0; by < M; by++){			//height
			
			unsigned curUpdVec = 0;
			//init
			uBlock(bx,by) = I(x+bx, y+by, z) - lambda0*update(x+bx, y+by, z);

			// Vectorial coordinate in F of the current pixel
			//curPixVec = M*(bx -1) + by;

			// Iterating to find contributions to the current pixel from the update of the rest
			// of the pixel in the block.
			for(unsigned bbx = 0; bbx < N; bbx++){		//width
				for(unsigned bby = 0; bby < M; bby++){	//height
					
					 
					uBlock(bx,by) += lambda0*update(x+bbx, y+bby, z)*F(curPixVec, curUpdVec);
					curUpdVec++;
				}
			}
			curPixVec++;
			// RECURSIVE STEP! Checking if the values are within bounds (min-max constraint)
			if(uBlock(bx,by)<mi || uBlock(bx,by)>Ma){
				minMaxViolated =  true;
				//counter++;
				lambda0 -= 0.1;
				SRconstraint(x,y,z);
				lambda0 = 1.0;
				break;
			}
		}
		if(minMaxViolated == true) break;
	}
	if(minMaxViolated == false){
		//min-max is not violated with the current lambda0, so data is stored in HR
//mexPrintf("in assignment part");
		for(unsigned bx = 0; bx < N; bx++){				//width
			for(unsigned by = 0; by < M; by++){			//height

				I(x+bx, y+by, z) = uBlock(bx, by);
			}
		}
	} 
}





/**********************************************************
 * The constructor for the SRsim class:		              *
 **********************************************************/
SRsim::SRsim(const CImg<float> & I_, const CImg<float> & bU_, const CImg<float> & fU_, 
		 double gamma_, double alpha_, 
		 unsigned fixpoint_, 
		 unsigned relax_, double convthres_,
		 unsigned lambdas_, unsigned lambdat_,
		 unsigned fixpointI_, unsigned relaxI_,
		 unsigned M_, unsigned N_, const CImg<float> & F_,
		 unsigned outerIte_)
		 : I(I_), bU(bU_), fU(fU_), gamma(gamma_), alpha(alpha_), 
		   fixpoint(fixpoint_), relax(relax_), convthres(convthres_), lambdas(lambdas_), lambdat(lambdat_),
		   fixpointI(fixpointI_), relaxI(relaxI_), M(M_), N(N_), F(F_), outerIte(outerIte_), eps2(1e-8)
{
	nx = I.width;
	ny = I.height;
	nz = I.depth;
	crelax = 0;
	cfix = 0;
	NaNdetected = false;
	A = CImg<float>(nx, ny, nz);
	update = Bf = Bb = A;
	lambda0 = 1.0;
	//setting min and max to allow for minor fluctuation around the uint8 intensity values
	//using [-1, 256] should be followed by a cast back to uint8 [0, 255]
	mi = -1.0;
	Ma = 256.0;
}
