/***************************************************************
 * Header file for Simultaneous flow and intensity SR          *
 *															   *
 * NOTE: Version for Mosix!!!!!!!!!!!!!!!!!!!!!!			   *
 ***************************************************************/
//For Unix use: MATLAB_MEX_FILE

#ifndef SRsim_H_LGJGH876876KJGHKJ7688
#define SRsim_H_LGJGH876876KJGHKJ7688

// for the matlab mex function, need the matlab plugin for cimg
#ifdef MATLAB_MEX_FILE
#include <mex.h>
#define cimg_plugin "cimgmatlabplugin.h" 
#endif
#define cimg_lapack 0

// if no matlab function these includes are used for the numerical exceptions
//#ifndef	MEXFILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
#include <string>
#include <sstream>
//#endif

#include <CImg.h>
using namespace cimg_library;


// exceptions only defined for non matlab version
//#ifndef MEX_FILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
class NumericalException {
protected:
	std::string value;
	static std::string toString(unsigned x) 
	{
		std::stringstream ss;
		ss << x;
		std::string res;
		ss >> res;
		return res;
	}
	std::string posString(unsigned x, unsigned y, unsigned z)
	{
		return "(" + toString(x) + "," + toString(y) + "," + toString(z) + ")";
	}
public:
	NumericalException() {}
	NumericalException(unsigned x, unsigned y, unsigned z) 
	{
		value = "Numerical problem at " + posString(x,y,z);
	}
	std::string Value() {return value;}
	virtual ~NumericalException() {}
};



class NaNException : NumericalException {
public:
	NaNException(unsigned x, unsigned y, unsigned z) 
	{
		value = "Got a NaN at " + posString(x, y, z);
	}
};

class INDException : NumericalException {
public:
	INDException(unsigned x, unsigned y, unsigned z)
	{
		value = "Got an indeterminate value at " + posString(x, y, z);
	}
};
//#endif

class SRsim {

    /* The algorithm variables */

	CImg<float> I;       // this is the HR image sequence
	CImg<float> U;       // this will be the flow container
	CImg<float> fU;       // this will be the forward flow in HR
	CImg<float> bU;       // this will be the backward flow in HR

	//flow variables
	double gamma;         // gradient constancy constraint weight;
	double alpha;         // smoothness weight;

	//unsigned levels;      // number of multiresolution levels;
	//double scalefactor;   // approximation of the scaling factor between consecutive levels

	unsigned fixpoint;    // number of fixed point iterations for the increment
	unsigned relax;       // maximum number of relaxation iterations in linear solver


	/* variables used in flow calc. */

	//CImg<float> Ir;      // spatially rescaled version of the image sequence I
	CImg<float> JU;      // fixed part (at a given level) for the flow Jacobian
	CImg<float> S;       // the tensor encoding the non linear combination of image derivative

	CImg<float> Data;    // the data coefficient
	CImg<float> Regul;   // the regularization coefficients;
	CImg<float> dU;      // the flow update

	// intensity variables
	double lambdas;					// spatial smoothness weight
	double lambdat;					// flow lines smoothess weight
	double lambda0;					// SR constraint weight
	double mi, Ma;					// min and max intensity values allowed

	unsigned fixpointI;				// number of fixed point iterations 
	unsigned relaxI;				// max. number of relaxation iterations for each fixed point iteration.

    unsigned M;						// height of the HR SR blocks
	unsigned N;						// width of the HR SR blocks
	CImg<float> F;					// the dicrect mapping/constraint/solution coeff. matrix

	CImg<float> A;					// spatial smoothness divergence coefs
	CImg<float> Bb,Bf;				// Flow smoothness divergence coefs.
	CImg<float> update;				// Reg. output, du, to be used in the actual recalc. (Ru'-u0)



	//general variables
	unsigned outerIte;		// no. of iterations of fU, bU and I runs 
	double convthres;		// threshold on difference of consecutive relaxation result for stopping


	unsigned clevel;      // current level in pyramid
	unsigned nx,ny,nz;    // size of the image sequence;
	unsigned cx, cy;      // current spatial size;

	unsigned crelax, cfix; //current iterations
	bool NaNdetected;	//debugging boolean to break

	const double eps2;    // normally 0.0001^2


public:
	SRsim(const CImg<float> & I_, const CImg<float> & bU_, const CImg<float> & fU_, 
		double gamma_, double alpha_,
		unsigned fixpoint_, 
		unsigned relax_, double convthres_,
		unsigned lambdas_, unsigned lambdat_,
		unsigned fixpointI_, unsigned relaxI_,
		unsigned M_, unsigned N_, const CImg<float> & F_,
		unsigned outerIte_);

	const CImg<float> & getSeq();   // returns sequence and is driver routine
	const CImg<float> & getfU();	// to enable return flows (and inspect/use them further)
	const CImg<float> & getbU();

private:
	// because it's usefull to have one:
	static double sqr(double x) {return x*x;}
	// 2x2 symmetric system solver A.X = B
	static void solve2x2(double a11, double a12, double a22, double b1, double b2, double & x, double & y); 
	// use my own sequence spatial resize; 
	//static void resize(CImg<float> & source, CImg<float> & dest);

//#ifndef MEXFILE_FUNCTION		//For Unix use: MATLAB_MEX_FILE
	// for debuging, test numerical values
	void checkValue(double val, unsigned x, unsigned y, unsigned z);
//#endif

	void computeIUDerivatives();   // Flow: Image derivatives and fixed part of the flow at a given level
	void fixedPointSolver();       // Flow: the fixed point solver for the flow update
	void computeEquationCoefs();   // Flow: data and regularization coefficients at each fixed point iteration
	void relaxationSolver();       // Flow: the linear solver at a given fixed point iteration

	void computeCoefficients();    // I: compute linear equation coefficients
	void linearSolver();		   // I: perform inpainting and denoising --> SR with lock to org. values
	void SRconstraint(unsigned x, unsigned y, unsigned z); // imposes the SR constraint as backprojection keeping minmax values

};




#endif // #ifndef SRsim_H_LGJGH876876KJGHKJ7688